@@ -1397,7 +1397,7 @@ extract.data <- function(z,what="data"){
   if (what=="residuals") {  
       if(!is.null(z$resscale)){
           ttt <- readBin(z$res,"integer",prod(z$dim),2)*z$resscale 
-          dim(ttt) <- z$dim
+          dim(ttt) <- z$dim[c(4,1:3)]
           } else {
           warning("extract.data: No residuals available, returning NULL")
           ttt <- NULL
