@@ -1,12 +1,12 @@
-#########################################################################################################################
+###################################################################################################
 #
-#    R - function  aws3D  for vector-valued  Adaptive Weights Smoothing (AWS) in 3D
+#    R - function  segm3D  for Adaptive Weights Segmentation in 3D SPM's
 #    
-#    reduced version for qtau==1, heteroskedastic variances only, exact value for Variance reduction
+#    exact value for Variance reduction from smoothed residuals
 #
 #    emaphazises on the propagation-separation approach 
 #
-#    Copyright (C) 2005 Weierstrass-Institut fuer
+#    Copyright (C) 2010-12 Weierstrass-Institut fuer
 #                       Angewandte Analysis und Stochastik (WIAS)
 #
 #    Author:  Joerg Polzehl
@@ -31,43 +31,28 @@ segm3D <- function(y,weighted=TRUE,
                    sigma2=NULL,mask=NULL,hinit=NULL,hmax=NULL,
                    ladjust=1,graph=FALSE,wghts=NULL,
                    df=100,h0=c(0,0,0),res=NULL, resscale=NULL, 
-                   ddim=NULL,delta=0,alpha=.05) {
+                   ddim=NULL,delta=0,alpha=.05,restricted=FALSE) {
 #
 #
 #  Auxilary functions
    IQRdiff <- function(y) IQR(diff(y))/1.908
-   getkrval <- function(df,ladj,n,kstar,alpha){
-#
-#    this delivers an upper bound for kritical values over a wide range of parameters
-#    covering the typical situations in fMRI
-#    n in (32^3 : 64^2*32)
-#    df in (40 : 250)
-#    ladj in (1 : 1.4) 
-#    kstar in (10:27)  corrsponding to maximal bandwidths  2.5 - 5
-#    see file sim_fmri_kritval.r in R/segmentation/fmrikrv/
-      dfinv <- log(df)-30000/df^1.44
-      explvar <- c(1, dfinv, log(n), kstar, dfinv*log(n), dfinv*(ladj-1))
-      a <- c(1, alpha, alpha^2, sqrt(alpha))
-      acoef <- matrix(c(1           ,  0           ,  0          ,  0           ,
-                        0.0002129201, -0.000820306 ,  0          ,  0           ,
-                        0.0016314442,  0.0061321878,  0          , -0.0043558989,
-                        0.0001558305,  0.0007918858,  0          , -0.0006520962,
-                       -0.0001535607,  0.0009159794, -0.004950741,  0           ,
-                        0.0003009761, -0.0017770459,  0.013396270,  0   ),4,6)
-      dimnames(acoef) <- list(c("(Intercept)","a","a2","ah"),NULL)
-      ecoefs <- t(acoef)%*%a
-      t(explvar)%*%ecoefs
-   }
-fovcorr <- function(h,fwhm=TRUE){
-# correction factor for FOV in case of spatial correlation
-fwhm2bw <- function(hfwhm) hfwhm/sqrt(8*log(2))
-if(fwhm) h <- fwhm2bw(h)
- 1-0.3181*log(h[1]+1)-0.3189*log(h[2]+1)+
-                       0.1025*log(h[1]+1)*log(h[2]+1)
-}
+   getkrval <- function(df,ladj,fov,k,alpha){
+      dfinv <- 1/df
+      lfov <- log(fov)
+      idffov <- dfinv*log(fov)
+      lalpha <- log(alpha)
+      lk <- log(k)
+      ladj <- ladj-1
+      kv <- 1.0105241 + 3.2036997*dfinv + 0.5442412*idffov -
+            0.0002686*lalpha -0.0172885*ladj +0.0009396*lk -1.7976621*dfinv*log(df) -
+            0.7856579*dfinv*lalpha+0.0765149*idffov*ladj
+      kv
+      }
 #
 # first check arguments and initialize
 #
+   args <- match.call()
+   if(is.null(ddim)) ddim <- dim(y)
 # test dimension of data (vector of 3D) and define dimension related stuff
    d <- 3
    dy <- dim(y)
@@ -85,18 +70,11 @@ if(fwhm) h <- fwhm2bw(h)
    lkern <- 1
    skern <- 1
 # define lambda
-   lambda <- ladjust*16
+   lambda <- ladjust*(exp(2.6-3.17*log(df)+8.4*log(log(df)))+16)
 # to have similar preformance compared to skern="Exp"
-   spmin <- .25 
-   spmax <- 1
    if (is.null(hinit)||hinit<1) hinit <- 1
-  
 # define hmax
    if (is.null(hmax)) hmax <- 5    # uses a maximum of about 520 points
-
-
-# define hincr
-
 # estimate variance in the gaussian case if necessary  
 # deal with homoskedastic Gaussian case by extending sigma2
    if (length(sigma2)==1) sigma2<-array(sigma2,dy[1:3]) 
@@ -109,30 +87,27 @@ if(fwhm) h <- fwhm2bw(h)
 # deal with homoskedastic Gaussian case by extending sigma2
    residuals <- readBin(res,"integer",prod(ddim),2)
   cat("\nfmri.smooth: first variance estimate","\n")
-  vartheta0 <- .Fortran("ivar",as.double(residuals),
+  varest0 <- .Fortran("ivar",as.double(residuals),
                            as.double(resscale),
                            as.logical(mask),
-                           as.integer(ddim[1]),
-                           as.integer(ddim[2]),
-                           as.integer(ddim[3]),
-                           as.integer(ddim[4]),
+                           as.integer(n1),
+                           as.integer(n2),
+                           as.integer(n3),
+                           as.integer(nt),
                            var = double(n1*n2*n3),
                            PACKAGE="fmri",DUP=FALSE)$var
-   varest0 <- vartheta0
    vq <- varest0*sigma2
-# Initialize  list for bi and theta
+#   plot(density(vq),main="Density of vq")
    if (is.null(wghts)) wghts <- c(1,1,1)
    hinit <- hinit/wghts[1]
    hmax <- hmax/wghts[1]
    wghts <- wghts[2:3]/wghts[1]
-#   tobj <- list(bi= rep(1,n))
    tobj <- list(bi= sigma2)
    theta <- y
    segm <- array(0,dy[1:3])
    varest <- varest0
    maxvol <- getvofh(hmax,lkern,wghts)
    fov <- sum(mask)
-   fov <- fovcorr(h0)*fov
    kstar <- as.integer(log(maxvol)/log(1.25))
    steps <- kstar+1
    k <- 1 
@@ -142,13 +117,13 @@ if(fwhm) h <- fwhm2bw(h)
    if (hinit>1) lambda0 <- 1e50 # that removes the stochstic term for the first step
    scorr <- numeric(3)
    total <- cumsum(1.25^(1:kstar))/sum(1.25^(1:kstar))
-   thresh <- 1
-   for(i in 10:kstar) thresh <- max(thresh,getkrval(df,ladjust,fov,i,alpha))
+#   for(i in 10:kstar) thresh <- max(thresh,tadj*getkrval(df,ladjust,fov,i,alpha))
+   thresh <- getkrval(df,ladjust,fov,kstar,alpha)
 #  just to ensure monotonicity of thresh with kmax, there exist a few parameter configurations
 #  where the approximation formula does not ensure monotonicity
    cat("FOV",fov,"delta",delta,"thresh",thresh,"ladjust",ladjust,"lambda",lambda,"df",df,"\n")
-# run single steps to display intermediate results
    residuals <- residuals*resscale
+   kv <- array(0,dy[1:3])
 #
 #   need these values to compute variances 
 #
@@ -158,7 +133,6 @@ if(fwhm) h <- fwhm2bw(h)
       cat("step",k,"bandwidth",signif(hakt,3)," ")
       dlw <- (2*trunc(hakt/c(1,wghts))+1)[1:d]
       hakt0 <- hakt
-      theta0 <- theta
       bi0 <- tobj$bi
       tobj <- .Fortran("segm3d",
                        as.double(y),
@@ -173,7 +147,7 @@ if(fwhm) h <- fwhm2bw(h)
                        as.double(df),
                        hakt=as.double(hakt),
                        as.double(lambda0),
-                       as.double(theta0),
+                       as.double(theta),
                        bi=as.double(bi0),
                        thnew=double(n1*n2*n3),
                        double(prod(dlw)),
@@ -183,12 +157,12 @@ if(fwhm) h <- fwhm2bw(h)
                        segm=as.integer(segm),
                        as.double(delta),
                        as.double(thresh),
-                       as.integer(k),
                        as.double(fov),
                        as.double(vq),
                        as.double(varest0),
                        varest=as.double(varest),
-                       PACKAGE="fmri",DUP=FALSE)[c("bi","thnew","hakt","segm","varest")]
+                       as.logical(restricted),
+                       PACKAGE="fmri",DUP=TRUE)[c("bi","thnew","hakt","segm","varest")]
       gc()
       theta <- array(tobj$thnew,dy[1:3]) 
       segm <- array(tobj$segm,dy[1:3])
@@ -209,11 +183,9 @@ if(fwhm) h <- fwhm2bw(h)
          cat(signif(total[k],2)*100,"%                 \r",sep="")
       }
       k <- k+1
-#  adjust lambda for the high intrinsic correlation between  neighboring estimates 
       lambda0 <- lambda
       gc()
    }
-
   z <- list(theta=theta,ni=tobj$bi,var=varest,y=y,segm=segm,
             hmax=tobj$hakt*switch(lkern,1,1,bw2fwhm(1/4)),
             scorr=scorr,mask=mask)
